function [ vysledek ] = em( data )
% 1 krok: data = csvread('height.csv.txt'); = inicializace dat
% 2 krok: em(data(:,1)) = dle zadani pouzijeme prvni sloupec dat
% 3 krok: dataplot (data) = vykresli graf
% dataplot(data, em(data(:,1))); = vykresli graf podle poctu iteraci

% inicializace pocatecnich parametru podle zadani ulohy
uMuz = 210.0;
uZena = 160.0;
oMuz = 10.0;
oZena = 10.0;
convergence = 0.00001;         % convergence = presnost algoritmu

nextStep = 1;
pocetIteraci = 0;

while nextStep == 1
    hodnotaM = uMuz;
    hodnotaZ = uZena;
    
    % pocet iteraci
    pocetIteraci = pocetIteraci + 1;  
    [uMuz, uZena, oMuz, oZena] = emIteration(data, uMuz, uZena, oMuz, oZena);
    if((abs(hodnotaM - uMuz) < convergence) && (abs(hodnotaZ - uZena) < convergence))
        nextStep = 0;
    end
    
    % nastaveni iteraci
    % if (pocetIteraci == 1)
    % nextStep = 0;
    % end
end
fprintf('Number of iterations: %d\n', pocetIteraci);
vysledek = [uMuz oMuz; uZena oZena];
return
end

% em algoritmus
function [uMuz, uZena, oMuz, oZena] = emIteration(data, uMuzIt, uZenaIt, oMuzIt, oZenaIt)
uMuz = uMuzIt;
uZena = uZenaIt;
oMuz = oMuzIt;
oZena = oZenaIt;

length = size(data, 1);  % pocet hodnot
probabilities1 = zeros(1, length);
probabilities2 = zeros(1, length);

% expectation krok - viz vzorec z prednasky
for i = 1:length
    temp1 = expectationForDataPart1(data(i), uMuz, oMuz);
    temp2 = expectationForDataPart1(data(i), uZena, oZena);
    probabilities1(i) = temp1 / (temp1 + temp2);
    probabilities2(i) = (1.0 - probabilities1(1, i));
end

% maximization krok - viz vzorec z prednasky
uMuz = (probabilities1 * data) / sum(probabilities1);       % nasobeni matic/sum
uZena = (probabilities2 * data) / sum(probabilities2);
oMuz = sqrt((probabilities1 * ((data - uMuz).^2.0)) / sum(probabilities1));
oZena = sqrt((probabilities2 * ((data - uZena).^2.0)) / sum(probabilities2));
return

end

function [ vysledek ] = expectationForDataPart1(data, mean, stdDev)
% pomocna fce pro vypocet expectation kroku
vysledek = (exp(-(((data - mean)^2.0) / (2.0 * stdDev^2.0))) * 0.5) / sqrt(2.0 * pi * stdDev);
return
end